ezInterrupt = LibStub("AceAddon-3.0"):NewAddon("ezInterrupt", "AceConsole-3.0", "AceEvent-3.0")

local init = false;
local outputChannel = nil;

local defaults = {
    profile = {
        enabled = true,
		announce = true,
		warning = true,
		whisper = true,
		whisperTarget = nil,
		custom = true,
		customChannel = nil,
		slashSay = true,
		slashParty = true,
		slashRaid = false,
		useInRaid = true,
		useInParty = true,
		useAlways = true,
		soundFile = "Sound\\Spells\\ShaysBell.ogg",
    },
}

local options = {
    name = "ezInterrupt Configuration",
    handler = ezInterrupt,
    type = "group",
    args = {
		enable = {
			type = "toggle",
			name = "Enable Addon",
			width = "full",
			set = "SetToggle",
			get = "GetToggle",
			order = 1
		},
		announce = {
			type = "toggle",
			name = "Announce Interrupts",
			width = "full",
			set = "SetAnnounce",
			get = "GetAnnounce",
			order = 2
		},
		warning = {
			type = "toggle",
			name = "Enable Warnings",
			width = "full",
			set = "SetWarning",
			get = "GetWarning",
			order = 3
		},
		output = {
			type = "group",
			name = "Announcement Options",
			order = 4,
			inline = true,
			args = {
				header1 = {
					type = "description",
					name = "Check the output channels that you want to make available to the addon. It will select the appropriate channel automatically and only report once per interrupt. Raid takes priority over Party which takes priority over Say.",
					order = 1
				},
				slashsay = {
					type = "toggle",
					name = "Say",
					width = "half",
					get = "GetSlashSay",
					set = "SetSlashSay",
					order = 4
				},
				slashparty = {
					type = "toggle",
					name = "Party",
					width = "half",
					get = "GetSlashParty",
					set = "SetSlashParty",
					order = 3
				},
				slashraid = {
					type = "toggle",
					name = "Raid",
					width = "half",
					get = "GetSlashRaid",
					set = "SetSlashRaid",
					order = 2
				},
				header2 = {
					type = "description",
					name = "In addition to the above, the addon can also whisper another player when you interrupt or show your interrupts in a custom channel.",
					order = 5
				},
				whisper = {
					type = "toggle",
					name = "Enable Whispering",
					set = "SetWhisper",
					get = "GetWhisper",
					order = 6
				},
				whispertarget = {
					type = "input",
					name = "Which Player?",
					desc = "Input the name of the player you want to whisper to.",
					get = "GetWhisperTarget",
					set = "SetWhisperTarget",
					order = 7
				},
				custom = {
					type = "toggle",
					name = "Enable Custom Channel",
					set = "SetCustom",
					get = "GetCustom",
					order = 8
				},
				customchannelfield = {
					type = "input",
					name = "Which Channel?",
					desc = "Insert the number of the custom channel you want to report to. Example: 5",
					get = "GetCustomChannel",
					set = "SetCustomChannel",
					order = 9
				}
			}
		},
		where = {
			type = "group",
			name = "Where do you want to use ezInterrupt?",
			order = 5,
			inline = true,
			args = {
				useinparty = {
					type = "toggle",
					name = "In party instances",
					width = "full",
					set = "SetUseInParty",
					get = "GetUseInParty",
					order = 2,
				},
				useinraid = {
					type = "toggle",
					name = "In raid instances",
					width = "full",					
					set = "SetUseInRaid",
					get = "GetUseInRaid",
					order = 2
				},
				usealways = {
					type = "toggle",
					name = "Everywhere",
					width = "full",
					set = "SetUseAlways",
					get = "GetUseAlways",
					order = 1
				},						
			}
		}
	}
}

function ezInterrupt:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("ezInterruptDB", defaults, "Default");
	LibStub("AceConfig-3.0"):RegisterOptionsTable("ezInterrupt", options);
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("ezInterrupt", "ezInterrupt");
	self.RegisterChatCommand("ezinterrupt", "ChatCommand");
	self.RegisterChatCommand("ezi", "ChatCommand");
	init = true;
end

-- debugging: chat commands are not working!
function ezInterrupt:ChatCommand(input)
    if not input or input:trim() == "" then
        InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
    else
        LibStub("AceConfigCmd-3.0").HandleCommand(ezInterrupt, "ezi", "ezInterrupt", input)
    end
end

function ezInterrupt:OnEnable()
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	if init then
		init = false;
	else
		UpdateOutputChannel(); -- sets the output channel according to player's instance and configuration settings
		UpdateCombatLogParsing(); -- enables or disables combat log parsing according to the player's configuration settings
	end
end
function ezInterrupt:OnDisable() -- AceEvent should turn off RegisterEvent automatically, but let's be sure
	self:UnregisterEvent("PLAYER_ENTERING_WORLD");
	self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
end

function ezInterrupt:PLAYER_ENTERING_WORLD(event)
	UpdateOutputChannel(); -- sets the output channel according to player's instance and configuration settings
	UpdateCombatLogParsing(); -- enables or disables combat log parsing according to the player's configuration settings
end

function UpdateOutputChannel()
	local instanceType = select(2, IsInInstance());
	if instanceType == "RAID" then
		if ezInterrupt.db.profile.slashRaid then outputChannel = "RAID";
		elseif ezInterrupt.db.profile.slashParty then outputChannel = "PARTY";
		elseif ezInterrupt.db.profile.slashSay then outputChannel = "SAY";
		else outputChannel = nil;
		end
	elseif instanceType == "PARTY" then
		if ezInterrupt.db.profile.slashParty then outputChannel = "PARTY";
		elseif ezInterrupt.db.profile.slashSay then outputChannel = "SAY";
		else outputChannel = nil;
		end
	elseif ezInterrupt.db.profile.slashSay then outputChannel = "SAY";
	else outputChannel = nil;
	end
end

function UpdateCombatLogParsing()
	local instanceType = select(2, IsInInstance());
	if ( instanceType == "RAID" and ezInterrupt.db.profile.useInRaid ) or ( instanceType == "PARTY" and ezInterrupt.db.profile.useInParty ) or ezInterrupt.db.profile.useAlways then
		ezInterrupt:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
	else
		ezInterrupt:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
	end
end

function InterruptReady() -- returns true if the player has usable interrupt abilities that are not on cooldown
	local class = UnitClass("player");	
	if class == "Warrior" then
		local available1, norage1 = IsUsableSpell("Pummel"); -- available1 = is Pummel available as ability to the player?   norage1 = is lack of rage making it unavailable?
		local cooldown1 = select(2, GetSpellCooldown("Pummel")); -- cooldown1 equals 0 if Pummel is not on cooldown
		local available2, norage2 = IsUsableSpell("Shield Bash"); 
		local cooldown2 = select(2, GetSpellCooldown("Shield Bash"));
		if ((available1 or norage1) and cooldown1 == 0) or ((available2 or norage2) and cooldown2 == 0) then
			return true;
		else
			return false;
		end
	elseif class == "Druid" then
		local available1, noenergy = IsUsableSpell(80965); -- Skull Bash (Cat)
		local cooldown1 = select(2, GetSpellCooldown(80965)); -- Skull Bash (Cat)
		local available2, norage = IsUsableSpell(80964); -- Skull Bash (Bear)
		local cooldown2 = select(2, GetSpellCooldown(80964)); -- Skull Bash (Bear)
		if ((available1 or noenergy) and cooldown1 == 0) or ((available2 or norage) and cooldown2 == 0) then
			return true;
		else
			return false;
		end
	elseif class == "Rogue" then
		local cooldown = select(2, GetSpellCooldown("Kick"));
		if cooldown == 0 then
			return true;
		else
			return false;
		end
	elseif class == "Paladin" then
		local cooldown = select(2, GetSpellCooldown("Rebuke"));
		if cooldown == 0 then
			return true;
		else
			return false;
		end
	elseif class == "Deathknight" then
		local cooldown = select(2, GetSpellCooldown("Mind Freeze"));
		if cooldown == 0 then
			return true;
		else
			return false;
		end
	elseif class == "Shaman" then
		local cooldown = select(2, GetSpellCooldown("Wind Shear"));
		if cooldown == 0 then
			return true;
		else
			return false;
		end
	elseif class == "Mage" then
		local cooldown = select(2, GetSpellCooldown("Counterspell"));
		if cooldown == 0 then
			return true;
		else
			return false;
		end
	elseif class == "Warlock" then
		local available, nomana = IsUsableSpell("Spell Lock");
		local cooldown = select(2, GetSpellCooldown("Spell Lock"));
		if (available or nomana) and cooldown == 0 then
			return true;
		else
			return false;
		end
	else
		return false;
	end
end

-- combat log parser
-- arguments passed are as follows:
-- self, timestamp, eventType, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags, spellID, spellName, spellSchool, extraSkillID, ...
function ezInterrupt:COMBAT_LOG_EVENT_UNFILTERED(_ , _, eventType, sourceGUID, sourceName, _, destGUID, destName, _, spellID, spellName, _, extraSkillID, ...)
	if self.db.profile.warning and eventType == "SPELL_CAST_START" and sourceGUID == UnitGUID("target") and UnitCanAttack("player", "target") then -- checks for spell casts made by the current target, if the player can attack it
		local unInterruptible = select(9, UnitCastingInfo("target")) -- checks if the cast is uninterruptible. 
		-- unInterruptible can be nil under certain circumstances
		if ( unInterruptible == false or unInterruptible ~= nil ) and InterruptReady() then 
			PlaySoundFile(self.db.profile.soundFile); -- this can be set to higher sound than current game sounds. check wowwiki for details
		end
	end
	if self.db.profile.announce and eventType == "SPELL_INTERRUPT" and sourceGUID == UnitGUID("player") then
		local spell = GetSpellLink(extraSkillID)
		if self.db.profile.whisper and self.db.profile.whisperTarget ~= nil then SendChatMessage("Interrupted " .. spell, "WHISPER", nil, self.db.profile.whisperTarget ); end
		if self.db.profile.custom and self.db.profile.customChannel ~= nil then SendChatMessage("Interrupted " .. spell, "CHANNEL", nil, self.db.profile.customChannel ); end	
		if outputChannel ~= nil then SendChatMessage("Interrupted " .. spell, outputChannel); end		
	end
end



-- functions for the Enable Addon checkbox
function ezInterrupt:SetToggle(info, val)
	self.db.profile.enabled = val;
	if self.db.profile.enabled then
		ezInterrupt:OnEnable();
	else
		ezInterrupt:OnDisable();
	end
end
function ezInterrupt:GetToggle(info, val)
	return self.db.profile.enabled;
end

-- functions for the Enable Reporting checkbox
function ezInterrupt:SetAnnounce(info, val)
	self.db.profile.announce = val;
end
function ezInterrupt:GetAnnounce(info, val)
	return self.db.profile.announce;
end

-- functions for the Enable Sound checkbox
function ezInterrupt:SetWarning(info, val)
	self.db.profile.warning = val;
end
function ezInterrupt:GetWarning(info)
	return self.db.profile.warning;
end

-- functions for the Say checkbox
function ezInterrupt:SetSlashSay(info, val)
	self.db.profile.slashSay = val;
	UpdateOutputChannel();
end
function ezInterrupt:GetSlashSay(info)
	return self.db.profile.slashSay;
end

-- functions for the Party checkbox
function ezInterrupt:SetSlashParty(info, val)
	self.db.profile.slashParty = val;
	UpdateOutputChannel();
end
function ezInterrupt:GetSlashParty(info)
	return self.db.profile.slashParty;
end

-- functions for the Raid checkbox
function ezInterrupt:SetSlashRaid(info, val)
	self.db.profile.slashRaid = val;
	UpdateOutputChannel();
end
function ezInterrupt:GetSlashRaid(info)
	return self.db.profile.slashRaid;
end

-- functions for the Enable Custom Channel checkbox
function ezInterrupt:SetCustom(info, val)
	self.db.profile.custom = val;
end
function ezInterrupt:GetCustom(info)
	return self.db.profile.custom;
end

-- functions for the Custom Channel field
function ezInterrupt:GetCustomChannel(info)
    return self.db.profile.customChannel;
end
function ezInterrupt:SetCustomChannel(info, val)
	if val == "" then
		self.db.profile.customChannel = nil;
	else
		self.db.profile.customChannel = val;
	end
end

-- functions for the Enable Whisper checkbox
function ezInterrupt:SetWhisper(info, val)
	self.db.profile.whisper = val;
end
function ezInterrupt:GetWhisper(info)
	return self.db.profile.whisper;
end

-- functions for the Whisper Target field
function ezInterrupt:GetWhisperTarget(info)
    return self.db.profile.whisperTarget;
end
function ezInterrupt:SetWhisperTarget(info, val)
	if val == "" then
		self.db.profile.whisperTarget = nil;
	else
		self.db.profile.whisperTarget = val;
	end
end

-- functions for the Use in Raid checkbox
function ezInterrupt:SetUseInRaid(info, val)
	self.db.profile.useInRaid = val;
	UpdateCombatLogParsing()
end
function ezInterrupt:GetUseInRaid(info)
	return self.db.profile.useInRaid;
end

-- functions for the Use in Party checkbox
function ezInterrupt:SetUseInParty(info, val)
	self.db.profile.useInParty = val;
	UpdateCombatLogParsing()
end
function ezInterrupt:GetUseInParty(info)
	return self.db.profile.useInParty;
end

-- functions for the Use Always checkbox
function ezInterrupt:SetUseAlways(info, val)
	self.db.profile.useAlways = val;
	UpdateCombatLogParsing()
end
function ezInterrupt:GetUseAlways(info)
	return self.db.profile.useAlways;
end